"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Check_1 = require("../types/Check");
const ObjectType_1 = require("../types/ObjectType");
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const constants_1 = require("../constants");
class Authorization {
    static check(checkParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const check = {
                warrants: [Check_1.checkWarrantParamsToCheckWarrant(checkParams)],
                debug: checkParams.debug
            };
            if (WarrantClient_1.default.config.authorizeEndpoint && options.warrantToken != "latest") {
                return this.makeEdgeCheckRequest(check, options);
            }
            return this.makeCheckRequest(check, options);
        });
    }
    static checkMany(checkParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const check = {
                op: checkParams.op,
                warrants: checkParams.warrants.map((checkWarrantParams) => Check_1.checkWarrantParamsToCheckWarrant(checkWarrantParams)),
                debug: checkParams.debug
            };
            if (WarrantClient_1.default.config.authorizeEndpoint && options.warrantToken != "latest") {
                return this.makeEdgeCheckRequest(check, options);
            }
            return this.makeCheckRequest(check, options);
        });
    }
    static batchCheck(checkParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const check = {
                op: Check_1.CheckOp.Batch,
                warrants: checkParams.warrants.map((checkWarrantParams) => Check_1.checkWarrantParamsToCheckWarrant(checkWarrantParams)),
                debug: checkParams.debug
            };
            return this.makeBatchCheckRequest(check, options);
        });
    }
    static hasFeature(featureCheckParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.check({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureCheckParams.featureId,
                },
                relation: "member",
                subject: featureCheckParams.subject,
                context: featureCheckParams.context,
                debug: featureCheckParams.debug
            }, options);
        });
    }
    static hasPermission(permissionCheckParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.check({
                object: {
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionCheckParams.permissionId,
                },
                relation: "member",
                subject: permissionCheckParams.subject,
                context: permissionCheckParams.context,
                debug: permissionCheckParams.debug
            }, options);
        });
    }
    // Private methods
    static makeCheckRequest(check, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/check`,
                data: check,
                options,
            });
            return response.code === 200;
        });
    }
    static makeEdgeCheckRequest(check, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    baseUrl: WarrantClient_1.default.config.authorizeEndpoint,
                    url: `/${constants_1.API_VERSION}/check`,
                    data: check,
                    options,
                });
                return response.code === 200;
            }
            catch (e) {
                if (e.code === "cache_not_ready") {
                    return this.makeCheckRequest(check);
                }
                throw e;
            }
        });
    }
    static makeBatchCheckRequest(check, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/check`,
                data: check,
                options,
            });
            return response.map((checkResult) => checkResult.code === 200);
        });
    }
}
exports.default = Authorization;
