"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const Permission_1 = __importDefault(require("./Permission"));
const PricingTier_1 = __importDefault(require("./PricingTier"));
const Role_1 = __importDefault(require("./Role"));
const Tenant_1 = __importDefault(require("./Tenant"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const ObjectType_1 = require("../types/ObjectType");
class User {
    constructor(userId, meta) {
        this.userId = userId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(params = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: params.userId,
                },
                meta: params.meta,
            }, options);
            return new User(response.objectId, response.meta);
        });
    }
    static batchCreate(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.batchCreate(params.map((user) => ({
                object: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: user.userId,
                },
                meta: user.meta,
            })), options);
            return response.map((object) => new User(object.objectId, object.meta));
        });
    }
    static get(userId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.get({
                objectType: ObjectType_1.ObjectType.User,
                objectId: userId,
            }, options);
            return new User(response.objectId, response.meta);
        });
    }
    static update(userId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.update({
                objectType: ObjectType_1.ObjectType.User,
                objectId: userId,
            }, { meta: params.meta }, options);
            return new User(response.objectId, response.meta);
        });
    }
    static delete(userId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.User,
                objectId: userId,
            }, options);
        });
    }
    static batchDelete(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield ObjectModule_1.default.batchDelete(params.map((user) => ({
                object: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: user.userId,
                },
            })), options);
        });
    }
    static listUsers(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.User }, listParams), options);
            const users = response.results.map((object) => new User(object.objectId, object.meta));
            return Object.assign(Object.assign({}, response), { results: users });
        });
    }
    static listUsersForTenant(tenantId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const queryResponse = yield WarrantModule_1.default.query(`select * of type user for tenant:${tenantId}`, listParams, options);
            const users = queryResponse.results.map((queryResult) => new User(queryResult.objectId, queryResult.meta));
            return Object.assign(Object.assign({}, queryResponse), { results: users });
        });
    }
    static assignUserToTenant(tenantId, userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removeUserFromTenant(tenantId, userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    //
    // Instance methods
    //
    listTenants(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Tenant_1.default.listTenantsForUser(this.userId, listParams, options);
        });
    }
    listRoles(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.listRolesForUser(this.userId, listParams, options);
        });
    }
    assignRole(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.assignRoleToUser(this.userId, roleId, options);
        });
    }
    removeRole(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.removeRoleFromUser(this.userId, roleId, options);
        });
    }
    listPermissions(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.listPermissionsForUser(this.userId, listParams, options);
        });
    }
    assignPermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.assignPermissionToUser(this.userId, permissionId, options);
        });
    }
    removePermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.removePermissionFromUser(this.userId, permissionId, options);
        });
    }
    hasPermission(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasPermission({ permissionId: params.permissionId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: params.context }, options);
        });
    }
    listPricingTiers(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.listPricingTiersForUser(this.userId, listParams, options);
        });
    }
    assignPricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.assignPricingTierToUser(this.userId, pricingTierId, options);
        });
    }
    removePricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.removePricingTierFromUser(this.userId, pricingTierId, options);
        });
    }
    listFeatures(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForUser(this.userId, listParams, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToUser(this.userId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromUser(this.userId, featureId, options);
        });
    }
    hasFeature(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: params.featureId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: params.context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.User;
    }
    getObjectId() {
        return this.userId;
    }
}
exports.default = User;
