"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const constants_1 = require("../constants");
const Object_1 = require("../types/Object");
const Warrant_1 = require("../types/Warrant");
class WarrantModule {
    static create(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/warrants`,
                data: {
                    objectType: Object_1.isWarrantObject(params.object) ? params.object.getObjectType() : params.object.objectType,
                    objectId: Object_1.isWarrantObject(params.object) ? params.object.getObjectId() : params.object.objectId,
                    relation: params.relation,
                    subject: Warrant_1.isSubject(params.subject) ? params.subject : { objectType: params.subject.getObjectType(), objectId: params.subject.getObjectId() },
                    policy: params.policy
                },
                options,
            });
        });
    }
    static batchCreate(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/warrants`,
                data: params.map((warrant) => ({
                    objectType: Object_1.isWarrantObject(warrant.object) ? warrant.object.getObjectType() : warrant.object.objectType,
                    objectId: Object_1.isWarrantObject(warrant.object) ? warrant.object.getObjectId() : warrant.object.objectId,
                    relation: warrant.relation,
                    subject: Warrant_1.isSubject(warrant.subject) ? warrant.subject : { objectType: warrant.subject.getObjectType(), objectId: warrant.subject.getObjectId() },
                    policy: warrant.policy,
                })),
                options,
            });
        });
    }
    static list(params = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.get({
                url: `/${constants_1.API_VERSION}/warrants`,
                params,
                options,
            });
        });
    }
    static query(query, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = Object.assign({}, listParams);
            // preserve backwards compatibility
            if (typeof query === "string") {
                params.q = query;
            }
            else {
                params.q = query.query;
                if (query.context) {
                    params.context = JSON.stringify(query.context);
                }
            }
            return yield WarrantClient_1.default.httpClient.get({
                url: `/${constants_1.API_VERSION}/query`,
                params: params,
                options,
            });
        });
    }
    static delete(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield WarrantClient_1.default.httpClient.delete({
                url: `/${constants_1.API_VERSION}/warrants`,
                data: {
                    objectType: Object_1.isWarrantObject(params.object) ? params.object.getObjectType() : params.object.objectType,
                    objectId: Object_1.isWarrantObject(params.object) ? params.object.getObjectId() : params.object.objectId,
                    relation: params.relation,
                    subject: Warrant_1.isSubject(params.subject) ? params.subject : { objectType: params.subject.getObjectType(), objectId: params.subject.getObjectId() },
                    policy: params.policy
                },
                options,
            });
            return response.warrantToken;
        });
    }
    static batchDelete(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield WarrantClient_1.default.httpClient.delete({
                url: `/${constants_1.API_VERSION}/warrants`,
                data: params.map((warrant) => ({
                    objectType: Object_1.isWarrantObject(warrant.object) ? warrant.object.getObjectType() : warrant.object.objectType,
                    objectId: Object_1.isWarrantObject(warrant.object) ? warrant.object.getObjectId() : warrant.object.objectId,
                    relation: warrant.relation,
                    subject: Warrant_1.isSubject(warrant.subject) ? warrant.subject : { objectType: warrant.subject.getObjectType(), objectId: warrant.subject.getObjectId() },
                    policy: warrant.policy,
                })),
                options,
            });
            return response.warrantToken;
        });
    }
}
exports.default = WarrantModule;
